/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.virtual.plugins.vfs.helpers;

import java.util.Collection;

import org.jboss.virtual.VirtualFile;
import org.jboss.virtual.VirtualFileFilter;

/**
 * Filters out a set of suffixes
 * 
 * @author adrian@jboss.org
 * @version $Revision: 44223 $
 */
public class SuffixesExcludeFilter implements VirtualFileFilter
{
   /** The suffixes */
   private Collection<String> suffixes;
   
   /**
    * Create a new SuffixMatchFilter,
    * 
    * @param suffixes the suffixes
    * @throws IllegalArgumentException for null suffixes
    */
   public SuffixesExcludeFilter(Collection<String> suffixes)
   {
      if (suffixes == null)
         throw new IllegalArgumentException("Null suffixes");
      for (String suffix : suffixes)
      {
         if (suffix == null)
            throw new IllegalArgumentException("Null suffix in " + suffixes);
      }
      this.suffixes = suffixes;
   }

   public boolean accepts(VirtualFile file)
   {
      String name = file.getName();
      for(String suffix : suffixes)
      {
         if (name.endsWith(suffix))
            return false;
      }
      return true;
   }
}
