package org.jboss.virtual.protocol.vfszip;

import org.jboss.virtual.protocol.AbstractVFSHandler;

/**
 * URLStreamHandler for VFS
 *
 * @author <a href="strukelj@parsek.net">Marko Strukelj</a>
 * @version $Revision: 1.0 $
 */
public class Handler extends AbstractVFSHandler
{
}
